@@ -5,17 +5,20 @@ import gradio.inputs as inputs
 import gradio.outputs as outputs
 import gradio.processing_utils
 import gradio.templates
-from gradio.blocks import Blocks, skip, update
+import gradio.themes as themes
+from gradio.blocks import Blocks
 from gradio.components import (
     HTML,
     JSON,
     Audio,
+    BarPlot,
     Button,
     Carousel,
     Chatbot,
     Checkbox,
     Checkboxgroup,
     CheckboxGroup,
+    Code,
     ColorPicker,
     DataFrame,
     Dataframe,
@@ -23,51 +26,67 @@ from gradio.components import (
     Dropdown,
     File,
     Gallery,
+    Highlight,
     Highlightedtext,
     HighlightedText,
     Image,
+    Interpretation,
     Json,
     Label,
+    LinePlot,
     Markdown,
     Model3D,
     Number,
     Plot,
     Radio,
+    ScatterPlot,
     Slider,
+    State,
     StatusTracker,
+    Text,
     Textbox,
     TimeSeries,
     Timeseries,
+    UploadButton,
     Variable,
     Video,
     component,
 )
-from gradio.examples import Examples
+from gradio.events import SelectData
+from gradio.exceptions import Error
 from gradio.flagging import (
     CSVLogger,
     FlaggingCallback,
+    HuggingFaceDatasetJSONSaver,
     HuggingFaceDatasetSaver,
     SimpleCSVLogger,
 )
+from gradio.helpers import EventData, Progress
+from gradio.helpers import create_examples as Examples
+from gradio.helpers import make_waveform, skip, update
 from gradio.interface import Interface, TabbedInterface, close_all
 from gradio.ipython_ext import load_ipython_extension
-from gradio.layouts import Box, Column, Group, Row, TabItem, Tabs
+from gradio.layouts import Accordion, Box, Column, Group, Row, Tab, TabItem, Tabs
 from gradio.mix import Parallel, Series
+from gradio.routes import Request, mount_gradio_app
 from gradio.templates import (
     Files,
-    Highlight,
+    ImageMask,
+    ImagePaint,
     List,
     Matrix,
     Mic,
     Microphone,
     Numpy,
+    Paint,
     Pil,
     PlayableVideo,
     Sketchpad,
-    Text,
     TextArea,
     Webcam,
 )
 
-current_pkg_version = pkgutil.get_data(__name__, "version.txt").decode("ascii").strip()
+current_pkg_version = (
+    (pkgutil.get_data(__name__, "version.txt") or b"").decode("ascii").strip()
+)
 __version__ = current_pkg_version
