@@ -1,6 +1,12 @@
+"""Contains methods that generate documentation for Gradio functions and classes."""
+
+from __future__ import annotations
+
 import inspect
+from typing import Callable, Dict, List, Tuple
 
 classes_to_document = {}
+classes_inherit_documentation = {}
 documentation_group = None
 
 
@@ -11,26 +17,62 @@ def set_documentation_group(m):
         classes_to_document[m] = []
 
 
-def document(*fns):
+def extract_instance_attr_doc(cls, attr):
+    code = inspect.getsource(cls.__init__)
+    lines = [line.strip() for line in code.split("\n")]
+    i = None
+    for i, line in enumerate(lines):
+        if line.startswith("self." + attr + ":") or line.startswith(
+            "self." + attr + " ="
+        ):
+            break
+    assert i is not None, f"Could not find {attr} in {cls.__name__}"
+    start_line = lines.index('"""', i)
+    end_line = lines.index('"""', start_line + 1)
+    for j in range(i + 1, start_line):
+        assert not lines[j].startswith("self."), (
+            f"Found another attribute before docstring for {attr} in {cls.__name__}: "
+            + lines[j]
+            + "\n start:"
+            + lines[i]
+        )
+    doc_string = " ".join(lines[start_line + 1 : end_line])
+    return doc_string
+
+
+def document(*fns, inherit=False):
     """
     Defines the @document decorator which adds classes or functions to the Gradio
     documentation at www.gradio.app/docs.
 
     Usage examples:
     - Put @document() above a class to document the class and its constructor.
-    - Put @document(fn1, fn2) above a class to also document the class methods fn1 and fn2.
+    - Put @document("fn1", "fn2") above a class to also document methods fn1 and fn2.
+    - Put @document("*fn3") with an asterisk above a class to document the instance attribute methods f3.
     """
 
     def inner_doc(cls):
         global documentation_group
+        if inherit:
+            classes_inherit_documentation[cls] = None
         classes_to_document[documentation_group].append((cls, fns))
         return cls
 
     return inner_doc
 
 
-def document_fn(fn):
-    doc_str = inspect.getdoc(fn)
+def document_fn(fn: Callable, cls) -> Tuple[str, List[Dict], Dict, str | None]:
+    """
+    Generates documentation for any function.
+    Parameters:
+        fn: Function to document
+    Returns:
+        description: General description of fn
+        parameters: A list of dicts for each parameter, storing data for the parameter name, annotation and doc
+        return: A dict storing data for the returned annotation and doc
+        example: Code for an example use of the fn
+    """
+    doc_str = inspect.getdoc(fn) or ""
     doc_lines = doc_str.split("\n")
     signature = inspect.signature(fn)
     description, parameters, returns, examples = [], {}, [], []
@@ -39,16 +81,20 @@ def document_fn(fn):
         line = line.rstrip()
         if line == "Parameters:":
             mode = "parameter"
-        elif line == "Example:":
+        elif line.startswith("Example:"):
             mode = "example"
+            if "(" in line and ")" in line:
+                c = line.split("(")[1].split(")")[0]
+                if c != cls.__name__:
+                    mode = "ignore"
         elif line == "Returns:":
             mode = "return"
         else:
             if mode == "description":
                 description.append(line if line.strip() else "<br>")
                 continue
-            assert line.startswith(
-                "    "
+            assert (
+                line.startswith("    ") or line.strip() == ""
             ), f"Documentation format for {fn.__name__} has format error in line: {line}"
             line = line[4:]
             if mode == "parameter":
@@ -73,7 +119,6 @@ def document_fn(fn):
         parameter_doc = {
             "name": param_name,
             "annotation": param.annotation,
-            "kind": param.kind.description,
             "doc": parameters.get(param_name),
         }
         if param_name in parameters:
@@ -82,6 +127,8 @@ def document_fn(fn):
             default = param.default
             if type(default) == str:
                 default = '"' + default + '"'
+            if default.__class__.__module__ != "builtins":
+                default = f"{default.__class__.__name__}()"
             parameter_doc["default"] = default
         elif parameter_doc["doc"] is not None and "kwargs" in parameter_doc["doc"]:
             parameter_doc["kwargs"] = True
@@ -103,7 +150,7 @@ def document_fn(fn):
 def document_cls(cls):
     doc_str = inspect.getdoc(cls)
     if doc_str is None:
-        return "", {}
+        return "", {}, ""
     tags = {}
     description_lines = []
     mode = "description"
@@ -142,7 +189,7 @@ def generate_documentation():
         documentation[mode] = []
         for cls, fns in class_list:
             fn_to_document = cls if inspect.isfunction(cls) else cls.__init__
-            _, parameter_doc, return_doc, _ = document_fn(fn_to_document)
+            _, parameter_doc, return_doc, _ = document_fn(fn_to_document, cls)
             cls_description, cls_tags, cls_example = document_cls(cls)
             cls_documentation = {
                 "class": cls,
@@ -155,13 +202,30 @@ def generate_documentation():
                 "fns": [],
             }
             for fn_name in fns:
-                fn = getattr(cls, fn_name)
-                (
-                    description_doc,
-                    parameter_docs,
-                    return_docs,
-                    examples_doc,
-                ) = document_fn(fn)
+                instance_attribute_fn = fn_name.startswith("*")
+                if instance_attribute_fn:
+                    fn_name = fn_name[1:]
+                    # Instance attribute fns are classes
+                    # whose __call__ method determines their behavior
+                    fn = getattr(cls(), fn_name).__call__
+                else:
+                    fn = getattr(cls, fn_name)
+                if not callable(fn):
+                    description_doc = str(fn)
+                    parameter_docs = {}
+                    return_docs = {}
+                    examples_doc = ""
+                    override_signature = f"gr.{cls.__name__}.{fn_name}"
+                else:
+                    (
+                        description_doc,
+                        parameter_docs,
+                        return_docs,
+                        examples_doc,
+                    ) = document_fn(fn, cls)
+                    override_signature = None
+                if instance_attribute_fn:
+                    description_doc = extract_instance_attr_doc(cls, fn_name)
                 cls_documentation["fns"].append(
                     {
                         "fn": fn,
@@ -171,7 +235,27 @@ def generate_documentation():
                         "parameters": parameter_docs,
                         "returns": return_docs,
                         "example": examples_doc,
+                        "override_signature": override_signature,
                     }
                 )
             documentation[mode].append(cls_documentation)
+            if cls in classes_inherit_documentation:
+                classes_inherit_documentation[cls] = cls_documentation["fns"]
+    for mode, class_list in classes_to_document.items():
+        for i, (cls, _) in enumerate(class_list):
+            for super_class in classes_inherit_documentation:
+                if (
+                    inspect.isclass(cls)
+                    and issubclass(cls, super_class)
+                    and cls != super_class
+                ):
+                    for inherited_fn in classes_inherit_documentation[super_class]:
+                        inherited_fn = dict(inherited_fn)
+                        try:
+                            inherited_fn["description"] = extract_instance_attr_doc(
+                                cls, inherited_fn["name"]
+                            )
+                        except (ValueError, AssertionError):
+                            pass
+                        documentation[mode][i]["fns"].append(inherited_fn)
     return documentation
