@@ -41,6 +41,7 @@ import fsspec.asyn
 import httpx
 import matplotlib.pyplot as plt
 import requests
+from huggingface_hub.utils import send_telemetry
 from markdown_it import MarkdownIt
 from mdit_py_plugins.dollarmath.index import dollarmath_plugin
 from mdit_py_plugins.footnote.index import footnote_plugin
@@ -57,6 +58,9 @@ if TYPE_CHECKING:  # Only import for type checking (is False at runtime).
 analytics_url = "https://api.gradio.app/"
 PKG_VERSION_URL = "https://api.gradio.app/pkg-version"
 JSON_PATH = os.path.join(os.path.dirname(gradio.__file__), "launches.json")
+GRADIO_VERSION = (
+    (pkgutil.get_data(__name__, "version.txt") or b"").decode("ascii").strip()
+)
 
 T = TypeVar("T")
 
@@ -113,7 +117,19 @@ def initiated_analytics(data: Dict[str, Any]) -> None:
         except (requests.ConnectionError, requests.exceptions.ReadTimeout):
             pass  # do not push analytics if no network
 
+    def initiated_telemetry_thread(data: Dict[str, Any]) -> None:
+        try:
+            send_telemetry(
+                topic="gradio/initiated",
+                library_name="gradio",
+                library_version=GRADIO_VERSION,
+                user_agent=data,
+            )
+        except Exception:
+            pass
+
     threading.Thread(target=initiated_analytics_thread, args=(data,)).start()
+    threading.Thread(target=initiated_telemetry_thread, args=(data,)).start()
 
 
 def launch_analytics(data: Dict[str, Any]) -> None:
@@ -130,6 +146,67 @@ def launch_analytics(data: Dict[str, Any]) -> None:
     threading.Thread(target=launch_analytics_thread, args=(data,)).start()
 
 
+def launched_telemetry(blocks: gradio.Blocks, data: Dict[str, Any]) -> None:
+    blocks_telemetry, inputs_telemetry, outputs_telemetry, targets_telemetry = (
+        [],
+        [],
+        [],
+        [],
+    )
+
+    from gradio.blocks import BlockContext
+
+    for x in list(blocks.blocks.values()):
+        blocks_telemetry.append(x.get_block_name()) if isinstance(
+            x, BlockContext
+        ) else blocks_telemetry.append(str(x))
+
+    for x in blocks.dependencies:
+        targets_telemetry = targets_telemetry + [
+            str(blocks.blocks[y]) for y in x["targets"]
+        ]
+        inputs_telemetry = inputs_telemetry + [
+            str(blocks.blocks[y]) for y in x["inputs"]
+        ]
+        outputs_telemetry = outputs_telemetry + [
+            str(blocks.blocks[y]) for y in x["outputs"]
+        ]
+    additional_data = {
+        "is_kaggle": blocks.is_kaggle,
+        "is_sagemaker": blocks.is_sagemaker,
+        "using_auth": blocks.auth is not None,
+        "dev_mode": blocks.dev_mode,
+        "show_api": blocks.show_api,
+        "show_error": blocks.show_error,
+        "theme": blocks.theme,
+        "title": blocks.title,
+        "inputs": blocks.input_components
+        if blocks.mode == "interface"
+        else inputs_telemetry,
+        "outputs": blocks.output_components
+        if blocks.mode == "interface"
+        else outputs_telemetry,
+        "targets": targets_telemetry,
+        "blocks": blocks_telemetry,
+        "events": [str(x["trigger"]) for x in blocks.dependencies],
+    }
+
+    data.update(additional_data)
+
+    def launched_telemtry_thread(data: Dict[str, Any]) -> None:
+        try:
+            send_telemetry(
+                topic="gradio/launched",
+                library_name="gradio",
+                library_version=GRADIO_VERSION,
+                user_agent=data,
+            )
+        except Exception as e:
+            print("Error while sending telemetry: {}".format(e))
+
+    threading.Thread(target=launched_telemtry_thread, args=(data,)).start()
+
+
 def integration_analytics(data: Dict[str, Any]) -> None:
     data.update({"ip_address": get_local_ip_address()})
 
@@ -141,7 +218,19 @@ def integration_analytics(data: Dict[str, Any]) -> None:
         except (requests.ConnectionError, requests.exceptions.ReadTimeout):
             pass  # do not push analytics if no network
 
+    def integration_telemetry_thread(data: Dict[str, Any]) -> None:
+        try:
+            send_telemetry(
+                topic="gradio/integration",
+                library_name="gradio",
+                library_version=GRADIO_VERSION,
+                user_agent=data,
+            )
+        except Exception as e:
+            print("Error while sending telemetry: {}".format(e))
+
     threading.Thread(target=integration_analytics_thread, args=(data,)).start()
+    threading.Thread(target=integration_telemetry_thread, args=(data,)).start()
 
 
 def error_analytics(message: str) -> None:
@@ -160,7 +249,19 @@ def error_analytics(message: str) -> None:
         except (requests.ConnectionError, requests.exceptions.ReadTimeout):
             pass  # do not push analytics if no network
 
+    def error_telemetry_thread(data: Dict[str, Any]) -> None:
+        try:
+            send_telemetry(
+                topic="gradio/error",
+                library_name="gradio",
+                library_version=GRADIO_VERSION,
+                user_agent=message,
+            )
+        except Exception as e:
+            print("Error while sending telemetry: {}".format(e))
+
     threading.Thread(target=error_analytics_thread, args=(data,)).start()
+    threading.Thread(target=error_telemetry_thread, args=(data,)).start()
 
 
 async def log_feature_analytics(feature: str) -> None:
@@ -268,7 +369,7 @@ def get_default_args(func: Callable) -> List[Any]:
 
 
 def assert_configs_are_equivalent_besides_ids(
-    config1: Dict, config2: Dict, root_keys: Tuple = ("mode", "theme")
+    config1: Dict, config2: Dict, root_keys: Tuple = ("mode",)
 ):
     """Allows you to test if two different Blocks configs produce the same demo.
 
@@ -276,7 +377,7 @@ def assert_configs_are_equivalent_besides_ids(
     config1 (dict): nested dict with config from the first Blocks instance
     config2 (dict): nested dict with config from the second Blocks instance
     root_keys (Tuple): an interable consisting of which keys to test for equivalence at
-        the root level of the config. By default, only "mode" and "theme" are tested,
+        the root level of the config. By default, only "mode" is tested,
         so keys like "version" are ignored.
     """
     config1 = copy.deepcopy(config1)
@@ -819,10 +920,14 @@ def check_function_inputs_match(fn: Callable, inputs: List, inputs_as_dict: bool
     """
 
     def is_special_typed_parameter(name):
+        from gradio.helpers import EventData
         from gradio.routes import Request
 
-        """Checks if parameter has a type hint designating it as a gr.Request"""
-        return parameter_types.get(name, "") == Request
+        """Checks if parameter has a type hint designating it as a gr.Request or gr.EventData"""
+        is_request = parameter_types.get(name, "") == Request
+        # use int in the fall-back as that will always be false
+        is_event_data = issubclass(parameter_types.get(name, int), EventData)
+        return is_request or is_event_data
 
     signature = inspect.signature(fn)
     parameter_types = typing.get_type_hints(fn) if inspect.isfunction(fn) else {}
@@ -832,7 +937,7 @@ def check_function_inputs_match(fn: Callable, inputs: List, inputs_as_dict: bool
     for name, param in signature.parameters.items():
         has_default = param.default != param.empty
         if param.kind in [param.POSITIONAL_ONLY, param.POSITIONAL_OR_KEYWORD]:
-            if not (is_special_typed_parameter(name)):
+            if not is_special_typed_parameter(name):
                 if not has_default:
                     min_args += 1
                 max_args += 1
