@@ -1,20 +1,18 @@
+import os
 import sys
-import unittest
 
 import pytest
 
 import gradio as gr
 
+os.environ["GRADIO_ANALYTICS_ENABLED"] = "False"
 
-class TestDocumentation(unittest.TestCase):
+
+class TestDocumentation:
     @pytest.mark.skipif(
         sys.version_info < (3, 8),
         reason="Docs use features in inspect module not available in py 3.7",
     )
-    def test_documentation(self):
+    def test_website_documentation(self):
         documentation = gr.documentation.generate_documentation()
         assert len(documentation) > 0
-
-
-if __name__ == "__main__":
-    unittest.main()
